;;; zre-mode.el --- Major mode for editing Zre code -*- lexical-binding: t; -*-
;;
;; Licensed under the GPLv3 License.

(defvar zre-keywords
  '("fn" "to" "in" "is" "as" "use" "if" "else"
    "for" "try" "catch" "while" "do" "exit"
    "return" "const" "type" "this" "yield"
    "break" "mod" "not" "and" "or" "print" "let")
  "Keywords in Zre.")

(defvar zre-types
  '("byte" "str" "real" "bool" 
    "int" "nat" "f32" "f64" 
    "u8" "u16" "u32" "u64" 
    "i8" "i16" "i32" "i64")
  "Types in Zre.")

(defvar zre-constants
  '("true" "false" "nil")
  "Constants in Zre.")

(defvar zre-font-lock-keywords
  `(
    ;; Keywords
    (,(regexp-opt zre-keywords 'words) . font-lock-keyword-face)

    ;; Types
    (,(regexp-opt zre-types 'words) . font-lock-type-face)

    ;; Constants
    (,(regexp-opt zre-constants 'words) . font-lock-constant-face)

    ;; Structs: CamelCase identifiers
    ("\\b[A-Z][a-zA-Z0-9_]*\\b" . font-lock-type-face)

    ;; Function definitions: "fn name(...)"
    ("\\_<fn\\>\\s-+\\(\\w+\\)(" (1 font-lock-function-name-face))

    ;; Function calls: "name(...)"
    ("\\_<\\([a-zA-Z_][a-zA-Z0-9_]*\\)\\s-*(" (1 font-lock-function-name-face))

    ;; Floating point numbers
    ("\\b-?[0-9]+\\.[0-9]+\\b" . font-lock-constant-face)

    ;; Integers
    ("\\b-?[0-9]+\\b" . font-lock-constant-face)

    ;; Strings with double quotes
    ("\"[^\"\n]*\"" . font-lock-string-face)

    ;; Strings with backticks
    ("`[^`]*`" . font-lock-string-face))

  "Font lock keywords for Zre mode.")
(defvar zre-mode-syntax-table
  (let ((st (make-syntax-table)))
    ;; C++-style comments
    (modify-syntax-entry ?/ ". 14b" st)
    ;; C-style comments
    (modify-syntax-entry ?* ". 23" st)
    ;; Strings
    (modify-syntax-entry ?\" "\"" st)
    (modify-syntax-entry ?` "\"" st)
    st)
  "Syntax table for Zre mode.")

(define-derived-mode zre-mode prog-mode "Zre"
  "Major mode for editing Zre code."
  :syntax-table zre-mode-syntax-table
  (setq-local font-lock-defaults '(zre-font-lock-keywords))
  (setq-local comment-start "// ")
  (setq-local comment-end "")
  (setq-local parse-sexp-ignore-comments t)
  (setq-local indent-line-function 'c-indent-line)
  (setq-local syntax-propertize-function
              (syntax-propertize-rules
               ;; Line comments starting with //
               ("\\(//\\)\\(?:[^\n]*\\)" (1 "<")))))

;;;###autoload
(add-to-list 'auto-mode-alist '("\\.zre\\'" . zre-mode))

(provide 'zre-mode)
;;; zre-mode.el ends here
